'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parser = parser;
exports.compiler = compiler;
exports.transformer = transformer;

var _babelCore = require('babel-core');

var _babylon = require('babylon');

var _asset = require('../asset.js');

function parser() {
  var code = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
  var asset = arguments[1];

  if (!code || code.length === 0) {
    asset.project.warn('attempt to parse empty asset: ' + asset.id);
  }

  try {
    return (0, _babylon.parse)(code, asset.parserOptions);
  } catch (error) {
    asset.project.error('Parser Error in Asset: ' + asset.id, error.message);
  }
}

function compiler(code, asset) {
  return transformer(code, asset).code;
}

function transformer(code, asset) {
  var id = asset.id;

  try {
    return (0, _babelCore.transform)(code, asset.transformerOptions || _asset.TransformerOptions);
  } catch (error) {
    return {
      ast: {},
      code: 'console.log(\'Error while compiling ' + id + '\', \'' + error.message + '\')',
      map: ''
    };
  }
}

function attemptRequire(moduleId) {
  try {
    return require(moduleId);
  } catch (error) {}
}