import DataSource from './data_source'

import clone from 'lodash/clone'
import omitBy from 'lodash/omitBy'

const EXTENSIONS = ['js', 'json', 'yaml', 'yml', 'csv']
const GLOB = '**/*.{' + EXTENSIONS.join(',') + '}'
const SECRET_REGEX = /(secret|secure|secured|hidden|private)/i

export class DataFile extends DataSource {

  static EXTENSIONS = EXTENSIONS;
  static GLOB = GLOB;

  static decorateCollection(collection) {
    defineProperty(collection, 'resolvePath', {
       enumerable: true,
       configurable: false,
       value: function(path){
         var parts = path.split('.')

         let match

         while(!match && parts.length > 0) {
           match = collection.at(parts.join('/'))
           parts.pop()
         }

         return match
       }
    })
  }

  constructor (uri, options = {}) {
    super(uri, options)

    this.indexer = parseSettings.bind(this)
  }

  set(key, value) {
    if (!value && isObject(key)) {
      Object.keys(key).forEach(k => {
        this.set(k, key[k])
      })
      return this
    }

    if(isString(key) && !key.match(SECRET_REGEX)) {
      set(this.parsed, key, value)
    }

    return this
  }
  /*
   * This lets us write less nested
   * versions of a settings file
   */
  getData() {
    if (this.extension !== '.js' && (!this.raw || this.raw.length === 0)) {
      this.readSync()
    }

    let value = this.indexed
    let parts = this.id.split('/')
    let last = parts[ parts.length - 1]

    /**
     * If the settings yaml has a single key whose value
     * is the name of the settings file, we will use the value
     * at that key instead of the redundant nesting
     */
    if (value[last] && keys(value).length === 1) {
      value = value[last] || {}
    }

    return value
  }
}

export default DataFile

function interpolateValues (obj, template) {
  let datasource = this

  Object.keys(obj)
  .filter(key => !key.match(SECRET_REGEX))
  .forEach(key => {
    let value = obj[key]

    if (typeof value === 'object') {
      interpolateValues.call(datasource, value, template)
    } else if (typeof value === 'string' && value.match(/^env\.\w+$/i)) {
      value = `<%= process.${ value } %>`
      obj[key] = template(value)(value)
    } else if (typeof value === 'string' ) {
      obj[key] = template(value)(value)
    }
  })

  return obj
}

function parseSettings (val = {}) {
  let base = interpolateValues.call(this, clone(val), this.templater.bind(this))

  return omitBy(base, (v,k) => {
     return k.match(SECRET_REGEX)
  })
}

const { defineProperty, keys } = Object
